# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import cerealizer
import soya, soya.cerealizer4soya, soya.sdlconst as sdlconst, soya.widget as widget
import jose3d
tofu = jose3d.tofu

from jose3d.character import Jose, MESSAGE_SOUND

ACTION_STOP_JUMPING = "j"

class Bonus(tofu.Mobile):
  TAKEN_SOUND = "ouais.wav"
  
  def __init__(self):
    tofu.Mobile.__init__(self)
    
    self.radius    = 1.0
    self.animation = 0
    self.solid     = 0
    
  def suitable_for(self, character): return isinstance(character, Jose)

  def get_network_state(self):
    if self.visible and not self.animation: return "1"
    else:                                   return "0"
    
  def read_network_state(self, f):
    if f.read(1) == "1": self.resurrected()
    else:                self.taken()

  def do_collisions(self):
    tofu.Mobile.do_collisions(self)
    
    if self.visible and not self.animation:
      for character in self.level.mobiles:
        if self.suitable_for(character) and self.distance_to(character.center) < self.radius:
          self.taken_by(character)
          self.set_current_state_importance(2)
          break
        
  def taken_by(self, character):
    character.send_message(MESSAGE_SOUND + self.TAKEN_SOUND)
    self.taken()
  
  def taken(self):
    self.animation = 1
    
  def resurrected(self):
    self.set_scale_factors(1.0, 1.0, 1.0)
    self.visible   = 1
    self.animation = 0
    
  def advance_time(self, proportion):
    if self.visible:
      if self.animation:
        self.rotate_lateral((0.2 * self.animation + 4.0) * proportion)
        
        self.animation += 1
        
        if   self.animation <  50.0: self.y += 0.15 * proportion
        elif self.animation <  80.0: pass
        elif self.animation < 100.0: self.scale(0.9, 0.9, 0.9)
        else:
          self.visible   = 0
          self.animation = 0
          
      else:
        self.rotate_lateral(4.0 * proportion)


class PropositionBonus(Bonus):
  def __init__(self, proposition = ""):
    super(PropositionBonus, self).__init__()
    
    self.proposition = proposition
    
    if   proposition.startswith(u"al" ): self.model = soya.Model.get("proposition")
    elif proposition.startswith(u"ump"): self.model = soya.Model.get("proposition@ump")
    elif proposition.startswith(u"ps" ): self.model = soya.Model.get("proposition@ps")
    
    self.sprite = soya.Sprite(self, soya.Material.get("x_lumiere_1"))
    self.sprite.width = self.sprite.height = 2.0
    self.sprite.lit = 0
    
  def taken_by(self, character):
    Bonus.taken_by(self, character)
    
    character.add_proposition(self.proposition)
    
cerealizer.register(PropositionBonus, soya.cerealizer4soya.SavedInAPathHandler(PropositionBonus))
