# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, os.path, random
import cerealizer
import soya, soya.cerealizer4soya, soya.sdlconst as sdlconst
import jose3d
tofu = jose3d.tofu
from jose3d.camera     import Camera


ACTION_MOVE_LEFT         = "<"
ACTION_MOVE_RIGHT        = ">"
ACTION_MOVE_UP           = "^"
ACTION_MOVE_DOWN         = "v"
ACTION_STOP_MOVING_LEFT  = "b"
ACTION_STOP_MOVING_RIGHT = "d"
ACTION_JUMP              = "J"
ACTION_STOP_JUMPING      = "j"
ACTION_VALIDATE          = "E"

MESSAGE_SOUND            = "S"
MESSAGE_ADD_PROPOSITION  = "P"
MESSAGE_DISCUSSION       = "D"
MESSAGE_HURT             = "H"
MESSAGE_HEAL             = "h"

class Character(tofu.SpeedInterpolatedMobile, tofu.AnimatedMobile, tofu.RaypickCollidedMobileWithGravity):
  gravity     = -0.02
  jump_speed  =  0.6
  max_y_speed =  0.5
  move_speed  =  0.2
  def __init__(self):
    super(Character, self).__init__()
    
    self.animable     = soya.World(self)
    self.animable.rotate_y(180.0)
    self.rotation     = 0.0
    self.raypick_dirs = [(self.left, 0), (self.up, 1)]
    self.invincible   = 0
    self.ejected      = 0
    self.life         = 1

  def set_default_animation(self):
    if   self.speed.y >  0.0: self.set_animation("saut")
    elif self.speed.y <  0.0: self.set_animation("chute")
    elif self.speed.z != 0.0: self.set_animation("marche2")
    else:                     self.set_animation("attente")
  def do_action(self, action):
    if self.ejected:
      self.set_animation("couic1")
      return
    
    animation = ""
    if    action == ACTION_VALIDATE:
      self.current_discussion.validate()
      
    elif  action == ACTION_MOVE_LEFT : self.speed.x = -self.move_speed; animation = "marche2"
    elif  action == ACTION_MOVE_RIGHT: self.speed.x =  self.move_speed; animation = "marche2"
    elif (action == ACTION_STOP_MOVING_LEFT ) and (self.speed.x < 0.0):
      self.speed.x = 0.0
      if not self.speed.z: animation = "attente"
      
    elif (action == ACTION_STOP_MOVING_RIGHT) and (self.speed.x > 0.0):
      self.speed.x = 0.0
      if not self.speed.z: animation = "attente"
    
    elif  action == ACTION_MOVE_UP:
      if (self.z > 0.0) and self.level.raypick_b(soya.Point(self.level, self.x - 0.5, 20.0, 0.0), self.down, -1, 1, 1) and self.level.raypick_b(soya.Point(self.level, self.x, 20.0, 0.0), self.down, -1, 1, 1) and self.level.raypick_b(soya.Point(self.level, self.x + 0.5, 20.0, 0.0), self.down, -1, 1, 1):
        self.speed.z = -self.move_speed
        animation = "marche2"
        
    elif  action == ACTION_MOVE_DOWN:
      if (self.z < 0.0) and self.level.raypick_b(soya.Point(self.level, self.x - 0.5, 20.0, 0.0), self.down, -1, 1, 1) and self.level.raypick_b(soya.Point(self.level, self.x, 20.0, 0.0), self.down, -1, 1, 1) and self.level.raypick_b(soya.Point(self.level, self.x + 0.5, 20.0, 0.0), self.down, -1, 1, 1):
        self.speed.z =  self.move_speed
        animation = "marche2"
        
    elif action == ACTION_JUMP:
      if self.speed.y == 0.0:
        self.play_sound("jump1.wav")
        self.speed.y = self.jump_speed
      
    elif action == ACTION_STOP_JUMPING:
      if self.speed.y > 0.0: self.speed.y = 0.0

    if   self.speed.y > 0.0: animation = "saut"
    elif self.speed.x < 0.0: animation = "chute"
    if animation: self.set_animation(animation)
    
    self.set_current_state_importance(2)

  def play_sound(self, sound): self.send_message(MESSAGE_SOUND + sound)
  
  def do_message(self, message):
    if   message[0] == MESSAGE_SOUND:
      if   jose3d.globdef.SOUND_SYSTEM == "openal":
        soya.SoundPlayer(self, soya.Sound.get(message[1:]), gain = 2.0)
      elif jose3d.globdef.SOUND_SYSTEM == "pymedia":
        import pymedia.player
        sound_player = pymedia.player.Player()
        sound_player.start()
        sound_player.startPlayback(os.path.join(jose3d.globdef.APPDIR, "sounds", message[1:]))
        jose3d.level.pymedia_sound_players.append(sound_player)
        
    else: raise ValueError("Unknown message code '%s'!" % message[0])
    
  def do_physics(self):
    old_speed_y = self.speed.y
    
    super(Character, self).do_physics()
    
    if (old_speed_y < 0.0) and (self.speed.y == 0.0): self.play_sound("chute.wav")
    
    if self.speed. x or self.speed.z:
      if self.ejected: ideal = soya.Vector(self.level, -self.speed.x, 0.0, self.speed.z + 0.05)
      else:            ideal = soya.Vector(self.level,  self.speed.x, 0.0, self.speed.z + 0.05)
      current = soya.Vector(self.animable, 0.0, 0.0, -1.0)
      
      angle = current.angle_to(ideal)
      #if current.cross_product(ideal).y < 0.0: angle = -angle
      
      if   angle >  15.0: self.rotation =  15.0
      elif angle < -15.0: self.rotation = -15.0
      else:               self.rotation = angle
      
    else:
      self.rotation = 0.0
      
    if    self.speed.y <  0.0:
      self.set_animation("chute")
      self.set_current_state_importance(2)
      
    elif (self.speed.y == 0.0) and (self.current_animation == "chute"):
      if self.speed.x or self.speed.z: self.set_animation("marche2")
      else:                            self.set_animation("attente")
      self.set_current_state_importance(2)
      
    if self.life > 0:
      if   (self.speed.z < 0.0) and self.next_state.z < -11.0:
        self.next_state.z = -11.0
        self.speed.z = 0.0
        if self.speed.x == 0.0: self.set_animation("attente")
        self.set_current_state_importance(2)
      elif (self.speed.z > 0.0) and self.next_state.z >  11.0:
        self.next_state.z =  11.0
        self.speed.z = 0.0
        if self.speed.x == 0.0: self.set_animation("attente")
        self.set_current_state_importance(2)
        
    if self.invincible:
      self.invincible -= 1
      if self.invincible % 2: self.visible = 0
      else:                   self.visible = 1
      
  def collide_wall(self, center, vec, collision, wall_normal):
    hypo = vec.length() * self.radius - center.distance_to(collision)
    wall_normal.__imul__(hypo)
    wall_normal.convert_to(self.level)
    wall_normal.z = 0.0
    return wall_normal
  
  def advance_time(self, proportion):
    super(Character, self).advance_time(proportion)
    
    self.animable.rotate_y(self.rotation * proportion)
    
  @tofu.side("client")
  def start_discussion(self, gettext_key): pass
  
  @tofu.side("single", "server")
  def start_discussion(self, gettext_key):
    self.send_message(MESSAGE_DISCUSSION + gettext_key.encode("utf8"))
    
    
    
class Jose(Character):
  def __init__(self):
    Character.__init__(self)
    
    self.current_discussion = None
    self.life               =  6
    self.last_ground_x      =  7.0
    self.last_ground_y      =  0.0
    self.last_ground_z      = 11.0
    
    self.animable.model = soya.AnimatedModel.get("jose")
    self.control_keys   = [sdlconst.K_LEFT, sdlconst.K_RIGHT, sdlconst.K_UP, sdlconst.K_DOWN, sdlconst.K_LSHIFT, sdlconst.K_RSHIFT]
    self.camera_target  = soya.Body(self)
    self.camera_target.set_xyz(2.0, 0.0, 0.0)
    
  def __getstate__(self):
    state = super(Jose, self).__getstate__()
    state[1]["current_discussion"] = None
    return state
    
  def add_proposition(self, proposition):
    self.play_sound("ouais.wav")
    self.send_message(MESSAGE_ADD_PROPOSITION + proposition)
    self.set_animation("bonus")
    self.set_current_state_importance(2)
    
  def has_proposition(self, proposition): return proposition in jose3d.player.Player.get(self.player_name).propositions
  
  def hurt(self, sens = 0):
    if (not self.invincible) and (self.life > 0):
      self.play_sound("couic1.wav")
      self.send_message(MESSAGE_HURT + str(sens))
      
  def do_message(self, message):
    if   message[0] == MESSAGE_ADD_PROPOSITION:
      proposition = message[1:]
      if not tofu.has_side("client"): jose3d.player.Player.get(self.player_name).propositions.add(proposition)
      self.start_discussion(u"__proposition_%s:%s__" % tuple(proposition.split(u":")))
      
    elif message[0] == MESSAGE_DISCUSSION:
      gettext_key = message[1:]
      if self.local:
        if gettext_key:
          characters = { "jose" : self }
          self.current_discussion = jose3d.discussion.Discussion(gettext_key, characters)
          self.current_discussion.show()
        elif self.current_discussion:
          self.current_discussion.hide()
          self.current_discussion = None
          
    elif message[0] == MESSAGE_HEAL:
      self.life = 6
      
    elif message[0] == MESSAGE_HURT:
      sens = float(message[1:])
      self.speed.x = 0.2 * sens
      self.life -= 1
      if self.life > 0:
        self.set_animation("couic1")
        self.invincible = 60
        self.ejected    = 20
      else:
        self.set_animation("couic2")
        self.invincible = 80
        self.ejected    = 80
        
    else: Character.do_message(self, message)
    
  def generate_actions(self):
    for event in soya.MAIN_LOOP.events:
      if   event[0] == sdlconst.KEYDOWN:
        if   (event[1] == sdlconst.K_q) or (event[1] == sdlconst.K_ESCAPE): soya.MAIN_LOOP.stop()
        elif event[1] == self.control_keys[0]: self.send_action(ACTION_MOVE_LEFT)
        elif event[1] == self.control_keys[1]: self.send_action(ACTION_MOVE_RIGHT)
        elif event[1] == self.control_keys[2]: self.send_action(ACTION_MOVE_UP)
        elif event[1] == self.control_keys[3]: self.send_action(ACTION_MOVE_DOWN)
        elif (event[1] == self.control_keys[4]) or (event[1] == self.control_keys[5]):
          if soya.MAIN_LOOP.bubble.discussion: self.send_action(ACTION_VALIDATE)
          else:                                self.send_action(ACTION_JUMP)
        elif event[1] == sdlconst.K_d:
          print self.x, self.y, self.z
          self.hurt()
          
        elif event[1] == sdlconst.K_c:
          import tempfile
          screenshot_file = tempfile.mktemp(".png")
          soya.screenshot(screenshot_file)
          print "* Jose3D * Screenshot saved as %s" % screenshot_file
          
      elif event[0] == sdlconst.KEYUP:
        if   event[1] == self.control_keys[0]: self.send_action(ACTION_STOP_MOVING_LEFT)
        elif event[1] == self.control_keys[1]: self.send_action(ACTION_STOP_MOVING_RIGHT)
        elif (event[1] == self.control_keys[4]) or (event[1] == self.control_keys[5]): self.send_action(ACTION_STOP_JUMPING)
        
      elif event[0] == sdlconst.JOYAXISMOTION:
        if   event[1] == 0:
          if   event[2] < 0: self.send_action(ACTION_MOVE_LEFT)
          elif event[2] > 0: self.send_action(ACTION_MOVE_RIGHT)
          elif self.speed.x < 0.0: self.send_action(ACTION_STOP_MOVING_LEFT)
          elif self.speed.x > 0.0: self.send_action(ACTION_STOP_MOVING_RIGHT)
          
        elif event[1] == 1:
          if   event[2] < 0: self.send_action(ACTION_MOVE_UP)
          elif event[2] > 0: self.send_action(ACTION_MOVE_DOWN)
          
      elif event[0] == sdlconst.JOYBUTTONDOWN:
        if event[1] == 0:
          if soya.MAIN_LOOP.bubble.discussion: self.send_action(ACTION_VALIDATE)
          else:                                self.send_action(ACTION_JUMP)
          
      elif event[0] == sdlconst.JOYBUTTONUP:
        if event[1] == 0: self.send_action(ACTION_STOP_JUMPING)
        
  def do_physics(self):
    super(Jose, self).do_physics()
    
    if self.ejected:
      self.ejected -= 1
      if self.ejected == 0:
        if self.current_animation == "couic1": self.set_animation("attente")
        self.speed.x = 0.0
      
  def do_collisions(self):
    if   (self.speed.x == 0.0) and (self.speed.y == 0.0) and (self.speed.z == 0.0) and (self.y > -15.0):
      self.last_ground_x = self.x
      self.last_ground_y = self.y
      self.last_ground_z = self.z
      
    elif self.y < -35.0:
      self.hurt()
      if self.life > 0: self.next_state.set_xyz(self.last_ground_x, self.last_ground_y + 8.0, self.last_ground_z)
      
    if (self.life <= 0) and (not self.invincible):
      self.killed()
      
  def advance_time(self, proportion):
    Character.advance_time(self, proportion)
    
    if   self.speed.x < 0.0:
      if self.camera_target.x > -2.0: self.camera_target.x -= 0.1 * proportion
    elif self.speed.x > 0.0:
      if self.camera_target.x <  2.0: self.camera_target.x += 0.1 * proportion
      
  def control_owned(self):
    super(Jose, self).control_owned()
    
    soya.MAIN_LOOP.life_bar.character = self
    soya.MAIN_LOOP.camera = Camera(soya.MAIN_LOOP.scenes[0])
    soya.MAIN_LOOP.camera.character = self
    soya.MAIN_LOOP.camera.back = 100.0
    soya.MAIN_LOOP.root_widget.insert(0, soya.MAIN_LOOP.camera)
    soya.widget.FPSLabel(soya.MAIN_LOOP.root_widget)
    
  def control_lost(self):
    super(Jose, self).control_lost()
    

cerealizer.register(Jose, soya.cerealizer4soya.SavedInAPathHandler(Jose))


class Nabot(Character):
  move_speed =  0.1
  jump_speed =  0.4
  def __init__(self):
    Character.__init__(self)
    
    self.animable.model = soya.AnimatedModel.get("nabot")
    self.radius     = 0.6
    self.radius_y   = 0.8
    self.center     = soya.Point(self, 0.0, self.radius_y, 0.0)
    self.state      = 0
    self.invincible = 40

  def hurt(self):
    self.play_sound("ecrase.wav")
    self.life -= 1
    Coup(self.level).set_xyz(self.x, self.y + 1.7, self.z + 0.5)
    
  def do_collisions(self):
    Character.do_collisions(self)
    
    for jose in self.level.mobiles:
      if isinstance(jose, Jose):
        d = self.distance_to(jose)
        if d < self.radius + jose.radius:
          if jose.y > self.y + self.radius_y * 0.5:
            if   self.speed.x >  0.1: self.speed.x =  0.1
            elif self.speed.x < -0.1: self.speed.x = -0.1
            jose.speed.y = jose.jump_speed * 0.6
            jose.set_animation("saut2")
            jose.set_current_state_importance(2)
            if (self.state == 0) or (self.state > 10): self.hurt()
            if self.state < 40: self.state = 1
          elif (self.state == 0) and (not self.invincible) and (not self.current_animation == "couic"):
            if self.x < jose.x: jose.hurt( 1)
            else:               jose.hurt(-1)
        elif d > 21.5:
          self.level.remove_mobile(self)
          return
        
    if self.state >= 1:
      self.state += 1
      self.set_animation("couic")
      
      if ((self.state == 40) or (self.life < -1)) and (self.speed.z == 0.0):
        if self.life > 0:
          self.state = 0
        else:
          self.speed.z = 0.15
          self.speed.y = 0.4
        
      if self.state > 100:
        self.level.remove_mobile(self)
        return
      
    if self.y < -20.0:
      self.level.remove_mobile(self)
      return
      
class Nabot1(Nabot):
  def __init__(self):
    Nabot.__init__(self)
    
    self.counter = 45
    self.sens = -1
    self.first_move = 1
    
  def generate_actions(self):
    if self.state == 0:
      self.counter -= 1
      
      if self.counter <= 20:
        if self.sens == -1: self.send_action(ACTION_STOP_MOVING_LEFT)
        else:               self.send_action(ACTION_STOP_MOVING_RIGHT)
        
        if self.counter <= 0:
          if self.first_move:
            self.first_move = 0
            self.counter = 65
          else:
            self.counter = 110
          self.sens *= -1
          
          if self.sens == -1: self.send_action(ACTION_MOVE_LEFT)
          else:               self.send_action(ACTION_MOVE_RIGHT)
          
cerealizer.register(Nabot1, soya.cerealizer4soya.SavedInAPathHandler(Nabot1))

class Nabot2(Nabot):
  move_speed =  0.2
  def __init__(self):
    Nabot.__init__(self)
    
    self.counter = 65
    self.sens = -1
    self.first_move = 1
    self.life = 2
    
    self.matraque = soya.Body(self.animable, soya.Model.get("matraque"))
    self.animable.attach_to_bone(self.matraque, "main_d")
    
  def hurt(self):
    Nabot.hurt(self)

    if self.matraque:
      ejected = Ejected(self.matraque.model)
      ejected.set_xyz(self.x, self.y + 0.5, self.z)
      self.level.add_mobile(ejected)
      
      self.animable.remove(self.matraque)
      self.matraque = None
      
      self.move_speed = 0.1
      
  def generate_actions(self):
    if self.state == 0:
      self.counter -= 1
      
      if (self.counter == 100) and self.matraque:
        self.send_action(ACTION_JUMP)
        
      if self.counter <= 20:
        if self.sens == -1: self.send_action(ACTION_STOP_MOVING_LEFT)
        else:               self.send_action(ACTION_STOP_MOVING_RIGHT)
        
        if self.counter <= 0:
          if self.first_move:
            self.first_move = 0
            self.counter = 65
          else:
            self.counter = 110
          self.sens *= -1
          
          if self.sens == -1: self.send_action(ACTION_MOVE_LEFT)
          else:               self.send_action(ACTION_MOVE_RIGHT)
          
cerealizer.register(Nabot2, soya.cerealizer4soya.SavedInAPathHandler(Nabot2))

class NabotGenerator(tofu.Mobile):
  def __init__(self, types = None, nb = 1):
    super(NabotGenerator, self).__init__()
    
    self.types = types or [1]
    self.nb    = nb
    self.time  = 0
    
  def do_collisions(self):
    self.time += 1

    if self.time > 90:
      nb = 0
      for character in self.level.mobiles:
        if   isinstance(character, Nabot) and (self.distance_to(character) < 15.0):
          nb += 1
          if nb >= self.nb:
            self.time -= 20
            return
        elif isinstance(character, Jose) and (self.distance_to(character) > 15.0):
          self.time -= 2
          return
        
      self.time = 0

      nabot_type = random.choice(self.types)
      if   nabot_type == 1: nabot = Nabot1()
      elif nabot_type == 2: nabot = Nabot2()
      nabot.set_xyz(self.x, self.y + 6.0, self.z)
      self.level.add_mobile(nabot)
      
      
cerealizer.register(NabotGenerator, soya.cerealizer4soya.SavedInAPathHandler(NabotGenerator))


class Gens(tofu.Mobile):
  def __init__(self, name = u"Gens10"):
    super(Gens, self).__init__()
    
    self.body = soya.Body(self)
    self.body.solid = 0
    self.body.rotate_y(180.0)
    
    self.wall = soya.Body(self, soya.Model.get("wall"))
    self.wall.visible = 0
    
    self.animable = None
    
    self.solid = 1
    self.state = 0
    
    self.name = name
    if name:
      self.body.model = soya.Model.get("gens%s@attente" % name[-1])
      
  def do_collisions(self):
    if self.state == 0:
      for jose in self.level.mobiles:
        if isinstance(jose, Jose):
          if self.distance_to(jose) < 3.0:
            jose.start_discussion("__%s__" % self.name)
            self.state = -1
            
    elif self.state == -1:
      for jose in self.level.mobiles:
        if isinstance(jose, Jose):
          if self.distance_to(jose) < 3.0: break
      else:
        self.state = 0
        
    #else:
      #if self.state < 50:
        #self.state += 1
        #if self.state == 50:
          #self.body.visible = 1
          #if self.animable:
          #  self.remove(self.animable)
          #  self.animable = None
            
  def proposition_found(self, jose):
    jose.play_sound("ouais.wav")
    jose.send_message(MESSAGE_HEAL)
    player = jose3d.player.Player.get(jose.player_name)
    player.restart_x = jose.x
    player.restart_y = jose.y
    player.restart_z = jose.z
    self.state = 1
    self.wall.solid = 0
    self.solid = 0
    
    self.body.model = soya.Model.get("gens%s@bonus" % self.name[-1])
    #self.body.visible = 0
    
    #self.animable = soya.Body(self, soya.AnimatedModel.get("gens%s" % self.name[-1]))
    #self.animable.rotate_y(180.0)
    #self.animable.solid = 0
    #self.animable.animate_blend_cycle("bonus")
    

    
cerealizer.register(Gens, soya.cerealizer4soya.SavedInAPathHandler(Gens))
    
    
class Coup(soya.Sprite):
  def __init__(self, parent):
    soya.Sprite.__init__(self, parent, soya.Material.get("coup"))
    self.color  = soya.WHITE #(1.0, 1.0, 1.0, 1.0)
    self.width  = self.height = 0.1
    self.solid  = 0
    self.time   = 0.0
    self.lit    = 0
    
  def begin_round(self):
    if self.time > 35.0:
      self.parent.remove(self)
      
  def advance_time(self, proportion):
    self.time += 3.0 * proportion
    if          self.time < 10.0:
      self.width = self.height = self.width + 0.2  * proportion
    elif 10.0 < self.time < 20.0:
      self.width = self.height = self.width - 0.15 * proportion
    elif 20.0 < self.time:
      self.width = self.height = self.width + 0.2  * proportion
      self.color = (1.0, 1.0, 1.0, 1.0 - (self.time - 20.0) / 15.0)
      
cerealizer.register(Coup)


class Ejected(tofu.SpeedInterpolatedMobile):
  def __init__(self, model):
    tofu.SpeedInterpolatedMobile.__init__(self)
    
    self.body = soya.Body(self, model)
    self.speed.set_xyz(random.uniform(-0.2, 0.2), random.uniform(0.25, 0.4), 0.08)
    self.rot_x = random.uniform(-5.0, 5.0)
    self.rot_y = random.uniform(-5.0, 5.0)
    
  def do_physics(self):
    tofu.SpeedInterpolatedMobile.do_physics(self)
    
    self.speed.y -= 0.01
    
  def do_collisions(self):
    if self.y < -35.0: self.level.remove_mobile(self)

  def advance_time(self, proportion):
    tofu.SpeedInterpolatedMobile.advance_time(self, proportion)
    
    self.body.turn_x(self.rot_x * proportion)
    self.body.turn_y(self.rot_y * proportion)
    
  
cerealizer.register(Ejected)


class Mouton(Character):
  gravity     = -0.012
  jump_speed  = 0.5
  
  def __init__(self):
    super(Mouton, self).__init__()
    
    #self.body = soya.Body(self, soya.AnimatedModel.get("mouton%s" % ("@noir" * noir)))
    if random.random() < 0.7: self.animable.model = soya.AnimatedModel.get("mouton@noir")
    else:                     self.animable.model = soya.AnimatedModel.get("mouton")
    
    self.animable.rotate_y(90.0)
    self.solid = 1
    self.counter = 0
    self.raypick_dirs = []

  def play_sound(self, sound): pass
  
  def do_physics(self):
    self.solid = 0
    super(Mouton, self).do_physics()
    self.solid = 1
    
  def generate_actions(self):
    self.counter += 1
    
    if self.counter == 200:
      self.counter = 0
      self.send_action(ACTION_JUMP)
      
    
cerealizer.register(Mouton, soya.cerealizer4soya.SavedInAPathHandler(Mouton))



class Panneau(tofu.Mobile):
  def __init__(self):
    tofu.Mobile.__init__(self)
    
    self.model = soya.Model.get("panneau")

  def do_collisions(self):
    for jose in self.level.mobiles:
      if isinstance(jose, Jose):
        if (jose.x > self.x) and (jose.z * self.z > 0.0):
          soya.MAIN_LOOP.stop("ok%s" % self.level.filename) # Don't work, since Player.logout also calls stop(), but without arg...
          soya.MAIN_LOOP.result = "ok%s" % self.level.filename
          
cerealizer.register(Panneau, soya.cerealizer4soya.SavedInAPathHandler(Panneau))
