# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import cerealizer
import soya, soya.widget as widget, soya.opengl as opengl
import jose3d
tofu = jose3d.tofu

from jose3d.discussion import Bubble
from jose3d.weather    import WeatherAtmosphere
from jose3d.level      import Level


def put_on_ground(obj, extra_y = 0.0):
  old_solid = obj.solid
  obj.solid = 0
  r = obj.get_root().raypick(obj, soya.Vector(obj.get_root(), 0.0, -1.0, 0.0))
  obj.solid = old_solid
  if r:
    obj.move(r[0])
    obj.y += extra_y
  

def create_levels():
  level = Level()
  level.static_part = static_part = soya.World(level)
  
  terrain = soya.Terrain(static_part)
  terrain.from_image(soya.Image.get("map1.png"))

  for x in range(0, 513):
    for z in range(0, 33):
      if terrain.get_height(x, z) < 0.27:
        terrain.set_vertex_option(x, z, 1, 1, 0)
  
  terrain.multiply_height(50.0)
  terrain.scale_factor = 1.0
  terrain.texture_factor = 1.0
  terrain.y = -35.0
  terrain.z = -17.0001
  #terrain.set_material_layer(soya.Material.get("grass2"),  0.0,  20.0)
  #terrain.set_material_layer(soya.Material.get("grass" ), 20.0,  30.0)
  
  terrain.set_material_layer(soya.Material.get("grass" ),  0.0,  30.0)
  #terrain.set_material_layer_angle(soya.Material.get("grass2"),  0.0,  30.0,    75.0, 180.0)
  
  terrain.set_material_layer(soya.Material.get("snow"  ), 30.0,  50.0)
  
  from jose3d.bonus import PropositionBonus
  from jose3d.character import Gens, NabotGenerator, Mouton, Panneau
  
  p = soya.Body(level, soya.Model.get("pancarte_pont"))
  p.set_xyz(29.0, 100.0, -11.0 - 1.0)
  put_on_ground(p, 0.0)
  
  p = soya.Body(level, soya.Model.get("pancarte_pont"))
  p.set_xyz(178.0, 100.0, -11.0 - 1.0)
  put_on_ground(p, 0.0)
  
  p = soya.Body(level, soya.Model.get("pancarte_pont"))
  p.set_xyz(373.0, 100.0, -11.0 - 1.0)
  put_on_ground(p, 0.0)
  
  
  p = PropositionBonus(u"al:38")
  level.add_mobile(p)
  p.set_xyz(3.0, 100.0, 11.0)
  put_on_ground(p, 1.5)
  
  g = Gens("Gens11")
  level.add_mobile(g)
  g.set_xyz(22.0, 100.0, 11.0 - 1.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"al:6")
  level.add_mobile(p)
  p.set_xyz(3.0, 100.0, -11.0)
  put_on_ground(p, 1.5)
  
  g = Gens("Gens12")
  level.add_mobile(g)
  g.set_xyz(62.0, 100.0, -11.0 - 1.0)
  put_on_ground(g)
  
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(70.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(98.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator(nb = 2)
  level.add_mobile(g)
  g.set_xyz(116.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(187.0, 100.0, -11.0)
  put_on_ground(g)
  
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(124.0, 100.0, -11.0)
  put_on_ground(v, 4.0)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(132.0, 100.0, -11.0)
  put_on_ground(v, 8.0)
  
  p = PropositionBonus(u"al:11")
  level.add_mobile(p)
  p.set_xyz(132.0, 100.0, -11.0)
  put_on_ground(p, 5.0)
  
  g = Gens("Gens14")
  level.add_mobile(g)
  g.set_xyz(197.0, 100.0, -11.0 - 1.0)
  put_on_ground(g)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(150.0, 100.0, 11.0)
  put_on_ground(v, 11.0)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(167.5, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(nb = 2)
  level.add_mobile(g)
  g.set_xyz(127.0, 100.0, 11.0)
  put_on_ground(g)


  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(111.0, -3.0, 11.0)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(101.0, -5.0, 11.0)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(96.0, 1.0, 11.0)
  
  g = Gens("Gens13")
  level.add_mobile(g)
  g.set_xyz(88.0, 100.0, 11.0 - 1.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(81.0, 100.0, 11.0)
  put_on_ground(g)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(65.0, -16.0, 11.0)
  
  v = soya.Body(level, soya.Model.get("champignon"))
  v.set_xyz(63.0, -9.0, 11.0)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(61.0, 100.0, 11.0)
  put_on_ground(g)

  g = NabotGenerator(nb = 2)
  level.add_mobile(g)
  g.set_xyz(43.0, 100.0, 11.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"al:15")
  level.add_mobile(p)
  p.set_xyz(37.0, 100.0, 11.0)
  put_on_ground(p, 2.0)
  
  v = soya.Body(level, soya.Model.get("ferme_facade"))
  v.set_xyz(259.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  v = soya.Body(level, soya.Model.get("ferme_facade"))
  v.set_xyz(283.0, 100.0, -11.0 + 1.0)
  put_on_ground(v, -1.0)
  
  v = soya.Body(level, soya.Model.get("ferme_facade"))
  v.set_xyz(308.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(325.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(346.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  g = NabotGenerator(nb = 2)
  level.add_mobile(g)
  g.set_xyz(262.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator(nb = 2)
  level.add_mobile(g)
  g.set_xyz(253.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(280.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(312.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(325.0, 100.0, -11.0)
  put_on_ground(g)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(336.0, 100.0, -11.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"al:26")
  level.add_mobile(p)
  p.set_xyz(353.0, -8.0, -11.0)
  put_on_ground(p, 1.5)
  
  g = Gens("Gens15")
  level.add_mobile(g)
  g.set_xyz(365.0, 100.0, -11.0 - 1.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"ump:1")
  level.add_mobile(p)
  p.set_xyz(399.0, 100.0, 11.0)
  put_on_ground(p, 3.0)
  
  p = PropositionBonus(u"al:75")
  level.add_mobile(p)
  p.set_xyz(202.0, 100.0, 11.0)
  put_on_ground(p, 2.5)
  
  g = Gens("Gens16")
  level.add_mobile(g)
  g.set_xyz(415.0, 100.0, 11.0 - 1.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(351.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(330.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [2])
  level.add_mobile(g)
  g.set_xyz(270.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(238.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = Gens("Gens17")
  level.add_mobile(g)
  g.set_xyz(208.0, 100.0, 11.0 - 1.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"ps:1")
  level.add_mobile(p)
  p.set_xyz(229.0, 100.0, 11.0)
  put_on_ground(p, 2.0)
  
  p = PropositionBonus(u"al:98")
  level.add_mobile(p)
  p.set_xyz(410.0, 100.0, 11.0)
  put_on_ground(p, 1.5)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(389.0, 100.0, -11.0)
  put_on_ground(g)
  
  m = Mouton()
  level.add_mobile(m)
  m.set_xyz(406.0, 100.0, -11.0)
  put_on_ground(m)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(413.0, 100.0, -11.0)
  put_on_ground(g)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(432.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, 0.0)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(430.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  m = Mouton()
  level.add_mobile(m)
  m.set_xyz(438.0, 100.0, -11.0)
  put_on_ground(m)
  
  g = NabotGenerator(types = [1, 2], nb = 2)
  level.add_mobile(g)
  g.set_xyz(446.0, 100.0, -11.0)
  put_on_ground(g)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(462.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -2.8)
  
  v = soya.Body(level, soya.Model.get("ferme_haute_facade"))
  v.set_xyz(460.0, 100.0, -11.0 - 1.0)
  put_on_ground(v, -1.0)
  
  m = Mouton()
  level.add_mobile(m)
  m.set_xyz(478.0, 100.0, -11.0)
  put_on_ground(m)
  
  m = Mouton()
  level.add_mobile(m)
  m.set_xyz(468.0, 100.0, -11.0)
  put_on_ground(m)
  
  g = NabotGenerator()
  level.add_mobile(g)
  g.set_xyz(460.0, 100.0, -11.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"al:1")
  level.add_mobile(p)
  p.set_xyz(427.0, 100.0, -11.0)
  put_on_ground(p, 2.0)
  
  g = Gens("Gens18")
  level.add_mobile(g)
  g.set_xyz(501.0, 100.0, -11.0 - 1.0)
  put_on_ground(g)
  
  p = PropositionBonus(u"al:121")
  level.add_mobile(p)
  p.set_xyz(509.0, 100.0, -11.0)
  put_on_ground(p, 3.0)
  
  g = Gens("Gens19")
  level.add_mobile(g)
  g.set_xyz(500.0, 100.0, 11.0 - 1.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [1, 2], nb = 2)
  level.add_mobile(g)
  g.set_xyz(443.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [2])
  level.add_mobile(g)
  g.set_xyz(478.0, 100.0, 11.0)
  put_on_ground(g)
  
  g = NabotGenerator(types = [1, 2])
  level.add_mobile(g)
  g.set_xyz(489.0, 100.0, 11.0)
  put_on_ground(g)
  
  p = Panneau()
  level.add_mobile(p)
  p.set_xyz(508.0, 100.0, 11.0 - 1.0)
  put_on_ground(p, -0.2)
  
  #soya.SoundPlayer(static_part, soya.Sound.get("alterbretonnade.ogg"), loop = 1, play_in_3D = 0)
  
  static_part.filename = "level1_static_part"; static_part.save()
  level      .filename = "level1"            ; level      .save()
  level.discard()
  
  return level
