# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import soya, soya.sdlconst as sdlconst, soya._soya, soya.widget as widget, soya.opengl as opengl
import jose3d
tofu = jose3d.tofu

class Bubble(widget.Label):
  def __init__(self, master = None, pos = 0, text = None, color = soya.BLACK):
    widget.Label.__init__(self, master, text, color = color)
    
    self.position   = soya.Point()
    self.discussion = None
    self.pos        = pos
    
    self.resize_style = [("percent left", 0.1), 'parent top', ('percent width', 0.8)]
    
  def build_display_list(self):
    opengl.glColor4f(*self._color)
    self._font.draw_area(self._text, self.left + 15, self.top + 8, 0.0, self.width - 30, self.height, self._align)
    
  def set_text(self, text):
    widget.Label.set_text(self, text)
    self.height = self.font.wordwrap(self._text, self.width - 30)[1] + 25.0
  text = property(widget.Label.get_text, set_text)
  
  def render(self):
    
    opengl.glEnable(opengl.GL_LINE_SMOOTH)
    
    if not self.discussion: return
    
    opengl.glPushMatrix()
    opengl.glTranslatef(self.left, self.top, 0.0)
    
    soya.DEFAULT_MATERIAL.activate()
    opengl.glEnable(opengl.GL_BLEND)
    opengl.glColor4f(1.0, 1.0, 1.0, 0.9)
    opengl.glBegin(opengl.GL_POLYGON)

    if self.pos < 2:
      opengl.glVertex2f(0.0, 0.0)
    else:
      opengl.glVertex2f(15.0             ,  0.0)
      opengl.glVertex2f(5.0              ,  5.0)
      opengl.glVertex2f(0.0              , 15.0)
      
    if self.pos == -1:
      opengl.glVertex2f(0.0              , self.height)
    else:
      opengl.glVertex2f(0.0              , self.height - 15.0)
      opengl.glVertex2f(5.0              , self.height -  5.0)
      opengl.glVertex2f(15.0             , self.height)
      
    if self.pos ==  1:
      opengl.glVertex2f(self.width       , self.height)
    else:
      opengl.glVertex2f(self.width - 15.0, self.height)
      opengl.glVertex2f(self.width - 5.0 , self.height -  5.0)
      opengl.glVertex2f(self.width       , self.height - 15.0)
      
    if self.pos < 2:
      opengl.glVertex2f(self.width, 0.0)
    else:
      opengl.glVertex2f(self.width       , 15.0)
      opengl.glVertex2f(self.width - 5.0 ,  5.0)
      opengl.glVertex2f(self.width - 15.0,  0.0)
      
    opengl.glEnd()

    character = getattr(self.discussion, "character", 0) or getattr(self, "character", 0)
    if character:
      opengl.glBegin(opengl.GL_TRIANGLES)
      if self.pos == 3:
        self.position.__init__(character, 0.0, 0.0, 0.0)
        x, y = soya.MAIN_LOOP.camera.coord3d_to_2d(self.position)
        x -= self.left
        y -= self.top
        
        opengl.glVertex2f(min(max(x - 5.0, 15.0), self.width - 25.0), 0.0)
        opengl.glVertex2f(min(max(x + 5.0, 25.0), self.width - 15.0), 0.0)
        opengl.glColor4f(1.0, 1.0, 1.0, 0.1)
        opengl.glVertex2f(x, y)
      else:
        self.position.__init__(character, 0.0, 2.0, 0.0)
        x, y = soya.MAIN_LOOP.camera.coord3d_to_2d(self.position)
        
        opengl.glVertex2f(self.width * 0.55, self.height)
        opengl.glVertex2f(self.width * 0.45, self.height)
        opengl.glColor4f(1.0, 1.0, 1.0, 0.1)
        opengl.glVertex2f(x - self.left, max(y, self.height + 20.0) - self.top)
      opengl.glEnd()
      
    opengl.glDisable(opengl.GL_BLEND)
        
    opengl.glPopMatrix()
    opengl.glColor4f(1.0, 1.0, 1.0, 1.0)
    
    widget.Label.render(self)


class Discussion(tofu.Multisided):
  def __init__(self, gettext_key, characters = {}, radius = 5.0):
    self.gettext_key = gettext_key
    self.characters  = characters
    self.text        = _(gettext_key)
    if self.text.startswith(u"__"): self.text = _(self.text)
    
    names = gettext_key[:-2].split("_")
    self.character = self._find_character(names[-1]) or self.character
    
  def _find_character(self, name):
    if   name == "None":                return None
    elif self.characters.has_key(name): return self.characters[name]
    else:
      for i in soya.MAIN_LOOP.scenes[0].recursive():
        if (getattr(i, "name", "") == name) or (i.__class__.__name__ == name): return i
    if name != "jose": return self._find_character("jose")
    
  def can_say(self):
    can_say = getattr(self, "can_say_" + self.gettext_key, None)
    if can_say and not can_say(): return 0
    
    prop = self.gettext_key[:-2].split("_")[-1]
    if ":" in prop:
      if not self.characters["jose"].has_proposition(prop): return 0
      
    return 1
  
  def show(self):
    if isinstance(self.character, jose3d.character.Gens):
      if u"al:" in self.gettext_key:
        self.character.proposition_found(self.characters["jose"])
        
    if not soya.MAIN_LOOP.camera: return
    
    self.bubble            = soya.MAIN_LOOP.bubble
    self.bubble.discussion = self
    self.bubble.text       = self.text# + u"\n       " + _("__press_key__")
    
  def hide(self):
    if self.bubble:
      self.bubble.discussion = None
      self.bubble = None
      
  def find_replies(self):
    if hasattr(self, "have_said_%s" % self.gettext_key): replies = getattr(self, "have_said_%s" % self.gettext_key)() or []
    else:                                                replies = []
    
    nb_ = self.gettext_key.count("_") + 1
    start = self.gettext_key[:-1]
    
    for key in jose3d.globdef.translator._catalog.keys():
      if key.startswith(start) and (key.count("_") == nb_):
        reply = Discussion(key, self.characters)
        if reply.can_say(): replies.append(reply)
        
    return replies
  
  @tofu.side("single", "client")
  def validate(self):
    self.hide()
    
  @tofu.side("single", "server")
  def validate(self):
    replies = self.find_replies()
    jose = self.characters["jose"]
    if replies:
      replies.sort(lambda a, b: cmp(a.gettext_key, b.gettext_key)) # Mets les "al" en premier
      jose.start_discussion(replies[0].gettext_key)
    else:
      jose.start_discussion(u"")
      
