# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2006-2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, random, math, shutil

import soya, soya.opengl  as soyaopengl
import soya.sdlconst      as soyasdlconst
import soya.widget        as widget
import jose3d
tofu = jose3d.tofu

import jose3d.globdef as globdef

COLOR     = (0.0, 0.0, 0.0, 1.0)
HIGHLIGHT = (1.0, 0.0, 0.0, 1.0)
CHOICELIST_RESIZE_STYLE = ('maximize', ('percent left', 0.28), ('margin bottom', 20), ('margin top', 20), ('margin left', 20))

def filter_string(s):
  return s.replace("\r", "").replace("\n", "")

class MainMenuMainLoop(soya.MainLoop):
  def __init__ (self, mainmenu):
    soya.MainLoop.__init__(self)
    self.mainmenu = mainmenu
    self.mainmenu.set_current_menu(self.mainmenu.game_menu_1)
    self.step     = 0
    
  def begin_round(self):
    soya.MainLoop.begin_round(self)
    if self.mainmenu.current_menu:
      for event in soya.process_event(): self.mainmenu.current_menu.process_event(event)

class Splash(soya.MainLoop):
  def __init__(self, *materials):
    soya.MainLoop.__init__(self)
    self.materials = materials
    self.i = 0
    self.state = 1
    
    self.root  = widget.Group()
    self.image = soya.widget.Image(self.root, materials[0], color = soya.BLACK)
    self.image.tex_bottom = 0.73828125
    self.image.resize_style = ('maximize', 'center x', 'center y')
    soya.set_root_widget(self.root)
    
  def begin_round(self):
    soya.MainLoop.begin_round(self)
    for event in soya.process_event():
      if (event[0] == soyasdlconst.KEYDOWN) or (event[0] == soyasdlconst.JOYBUTTONDOWN):
        self.state = -1
        
  def advance_time(self, proportion):
    x = self.image.color[0] + self.state * proportion * 0.03
    if   x > 1.0:
      self.image.color = soya.WHITE
    elif x < 0.0:
      self.image.color = soya.BLACK
      if self.i + 1 < len(self.materials):
        self.i += 1
        self.image.material = self.materials[self.i]
        self.state = 1
      else:
        self.stop()
    else:
      self.image.color = (x, x, x, 1.0)

  

class MainMenu:
  def __init__(self):
    self.current_menu = None
    self.old_menus = []
    
    self.root = widget.Group()

    self.bg_img = soya.widget.Image(self.root, soya.Material.get('intro'))
    self.bg_img.resize_style  = ('maximize', 'center x', 'center y')
    self.bg_img.tex_bottom = 0.73828125
    
    version_label = soya.widget.Label(self.root, _("Version") + " " + globdef.VERSION)
    version_label.resize_style = ('maximize width', ('margin left', 5))
    
    self.scene = soya.World()
    
    self.atm = soya.NoBackgroundAtmosphere()
    self.atm.ambient  = (0.2, 0.2, 0.2, 1.0)
    self.scene.atmosphere = self.atm
    
    self.game_menu_1 = soya.widget.ChoiceList(self.root, [
      soya.widget.Choice(_('Play'   ), self.show_solo),
      soya.widget.Choice(_('Options'), self.show_h_option),
      soya.widget.Choice(_('Sign'   ), self.sign),
      soya.widget.Choice(_('Quit'   ), self.go_back)
      ], None, COLOR, HIGHLIGHT, cancel = -1)
    self.game_menu_1.resize_style = CHOICELIST_RESIZE_STYLE
    self.game_menu_1.visible = 0
    
    checkbox = [_('off'), _('on')]
    
    self.menu_h_option = soya.widget.ChoiceList(self.root, [
      soya.widget.Choice(_('Resolution'), None, None, ['320x240', '640x480', '800x600', '1024x768', '1152x864', '1280x800', '1280x1024', '1400x1050', '1680x1050']),
      soya.widget.Choice(_('Fullscreen'), None, None, checkbox),
      soya.widget.Choice(_('Graphic quality'), None, None, [_('Low'), _('Medium'), _('High')]),
      soya.widget.Choice(_('Sound volume'), None, None, ['0', '10', '20', '30', '40', '50', '60', '70', '80', '90', '100']),
      soya.widget.ChoiceInput(_('Saved game directory')),
      soya.widget.Choice(_('Back'), self.back_h_option)
      ], None, COLOR, HIGHLIGHT, cancel = -1)
    self.menu_h_option.resize_style = CHOICELIST_RESIZE_STYLE
    self.menu_h_option.visible = 0
    
    self.label = widget.Group(self.root)
    self.label.visible = 0
    self.label.process_event = lambda event: self.menu_label.process_event(event)
    
    self.menu_label = soya.widget.ChoiceList(self.label, [
      soya.widget.Choice( u""     , None, None, [""]),
      soya.widget.Choice(_('Back'), self.go_back),
      ], None, COLOR, HIGHLIGHT, cancel = -1)
    self.menu_label.resize_style = CHOICELIST_RESIZE_STYLE
    
    self.solo = widget.Group(self.root)
    self.solo.visible = 0
    self.solo.process_event = lambda event: self.menu_solo.process_event(event)
    menu_items = [
      soya.widget.Choice(_('__level_1__'   ), self.start_level_1),
      soya.widget.Choice(_('Back'          ), self.go_back),
      ]
      
    self.menu_solo = soya.widget.ChoiceList(self.solo, [
      soya.widget.Choice(_('__level_1__'   ), self.start_level_1),
      soya.widget.Choice(_('Back'          ), self.go_back),
      ], None, COLOR, HIGHLIGHT, cancel = -1)
    self.menu_solo.resize_style = CHOICELIST_RESIZE_STYLE
    
  def set_current_menu(self, menu):
    if self.current_menu:
      self.current_menu.visible = 0
      self.old_menus.append(self.current_menu)
    self.current_menu = menu
    self.current_menu.visible = 1
    
  def go_back(self):
    if len(self.old_menus) == 0:
      globdef.generate_config_file()
      soya.MAIN_LOOP.stop()
      import sys
      sys.exit()
    if self.current_menu:
      self.current_menu.visible = 0
    self.current_menu = self.old_menus.pop()
    self.current_menu.visible = 1
    
  def show_h_option(self):
    self.menu_h_option.choices[0].value = str(globdef.SCREEN_WIDTH) + 'x' + str(globdef.SCREEN_HEIGHT)
    self.menu_h_option.choices[1].value = self.menu_h_option.choices[1].range[globdef.FULLSCREEN]
    self.menu_h_option.choices[2].value = self.menu_h_option.choices[2].range[globdef.QUALITY]
    #self.menu_h_option.choices[3].value = self.menu_h_option.choices[3].range[globdef.SOUND]
    self.menu_h_option.choices[3].value = str(int(round(globdef.SOUND_VOLUME * 100)))
    self.menu_h_option.choices[4].value = tofu.SAVED_GAME_DIR
    self.set_current_menu(self.menu_h_option)
    
  def back_h_option(self):
    resa = self.menu_h_option.choices[0].value
    globdef.SCREEN_WIDTH     = int(resa[: resa.find("x")])
    globdef.SCREEN_HEIGHT    = int(resa[resa.find("x") + 1:])
    globdef.FULLSCREEN       = (self.menu_h_option.choices[1].value == _('on'))
    globdef.QUALITY          = self.menu_h_option.choices[2].range.index(self.menu_h_option.choices[2].value)
    globdef.SOUND_VOLUME     = int(self.menu_h_option.choices[3].value) / 100.0
    tofu.SAVED_GAME_DIR  = self.menu_h_option.choices[4].value
    soya.set_quality(globdef.QUALITY)
    soya.set_video(globdef.SCREEN_WIDTH, globdef.SCREEN_HEIGHT, globdef.FULLSCREEN, 1)
    if globdef.SOUND_SYSTEM == "openal": soya.set_sound_volume(globdef.SOUND_VOLUME)
    self.go_back()
    
  def sign(self):
    if globdef.FULLSCREEN:
      soya.set_video(globdef.SCREEN_WIDTH, globdef.SCREEN_HEIGHT, 0, 1)
    import webbrowser
    webbrowser.open("http://www.unisavecbove.org")
    
  def check_saved_game(self):
    has_saved_game = os.path.exists(os.path.join(tofu.SAVED_GAME_DIR, globdef.LOGIN))
    if        has_saved_game  and (len(self.menu_solo.choices) == 2):
      self.menu_solo.choices.insert(0, soya.widget.Choice(_('Load last game'), self.load_last_game))
    elif (not has_saved_game) and (len(self.menu_solo.choices) == 3):
      del self.menu_solo.choices[0]
      
  def show_solo(self):
    self.check_saved_game()
    self.set_current_menu(self.solo)

  def splash(self, *materials):
    main_loop = soya.MAIN_LOOP
    Splash(*materials).main_loop()
    soya.set_root_widget(self.root)
    main_loop.reset()
    
  def start_level_1(self):
    #self.splash(soya.Material.get("ecran_debut_1")).main_loop()
    self.splash(soya.Material.get("ecran_debut"), soya.Material.get("ecran_debut_1"))
    self.play_solo(1)
  
  def load_last_game(self):
    self.play_solo(None)
  
  def play_solo(self, level):
    tofu.GAME = globdef.LOGIN
    if not level is None:
      try:
        shutil.rmtree(os.path.join(tofu.SAVED_GAME_DIR, tofu.GAME))
      except: pass
      
    main_loop = soya.MAIN_LOOP
    tofu.PLAYER_IDS = [jose3d.player.PlayerID(globdef.LOGIN, "test", level)]

    tofu.set_side("single")
    
    try:
      m = jose3d.level.MainLoop()
      m.result = None # Hack
      r = m.main_loop()
      r = m.result # Hack
    except:
      error, value, traceback = sys.exc_info()
      sys.excepthook(error, value, traceback)
      r = "%s(%s)" % (error, value)
      
    self.check_saved_game()
    self.menu_solo.selected = 0
    soya.set_root_widget(self.root)
    main_loop.reset()
    soya.MAIN_LOOP = main_loop
    
    if r == "oklevel1":
      self.splash(soya.Material.get("ecran_fin_1"), soya.Material.get("ecran_fin"), soya.Material.get("ecran_fin2"))
      
    elif r: self.message("Error: " + str(r))
    
  def message(self, text):
    self.menu_label.choices[0].label = text
    self.set_current_menu(self.label)
    


  def run (self):
    soya.set_root_widget(self.root)
    self.main_loop = MainMenuMainLoop(self)
    action = self.main_loop.main_loop()
  









