# -*- coding: utf-8 -*-

# Jose3D
# Copyright (C) 2007 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, os.path
import cerealizer
import soya, soya.widget as widget, soya.opengl as opengl
import jose3d
tofu = jose3d.tofu

from jose3d.discussion import Bubble
from jose3d.weather    import WeatherAtmosphere


class LifeBar(widget.Widget):
  def __init__(self, master, character = None):
    widget.Widget.__init__(self, master)
    self.character = character
    self.material  = soya.Material.get("coeur")
    
  def render(self):
    if self.character:
      self.material.activate()
      
      opengl.glEnable(opengl.GL_BLEND)
      opengl.glBegin(opengl.GL_QUADS)
      opengl.glTexCoord2f(0.0, 0.0                      ); opengl.glVertex2f( 0.0,  0.0)
      opengl.glTexCoord2f(1.0, 0.0                      ); opengl.glVertex2f(50.0,  0.0)
      opengl.glTexCoord2f(1.0, 0.5 * self.character.life); opengl.glVertex2f(50.0, 25.0 * self.character.life)
      opengl.glTexCoord2f(0.0, 0.5 * self.character.life); opengl.glVertex2f( 0.0, 25.0 * self.character.life)
      opengl.glEnd()
      opengl.glDisable(opengl.GL_BLEND)
      
    

music_player = None
pymedia_sound_players = []

class MainLoop(tofu.MainLoop):
  def init_interface(self):
    self.root_widget = widget.Group()
    soya.set_root_widget(self.root_widget)
    self.life_bar = LifeBar(self.root_widget)
    self.bubble = Bubble(self.root_widget, 0, "test -1", (0.0, 0.0, 0.0, 1.0))
    
    self.scenes[0].atmosphere = WeatherAtmosphere(self.scenes[0])
    
    if   jose3d.globdef.SOUND_SYSTEM == "openal":
      soya.SoundPlayer(self.scenes[0], soya.Sound.get("alterbretonnade.ogg"), loop = 1, play_in_3D = 0)
      
    elif jose3d.globdef.SOUND_SYSTEM == "pymedia":
      global music_player
      import pymedia.player
      music_player = pymedia.player.Player()
      music_player.start()
      music_player.startPlayback(os.path.join(jose3d.globdef.APPDIR, "sounds", "alterbretonnade.ogg"))
      music_player.setLoops(100000)
      jose3d.level.pymedia_sound_players.append(music_player)

  def stop(self, *args):
    if music_player: music_player.stop()
    tofu.MainLoop.stop(self, *args)
    
  def begin_round(self):
    self.events = soya.process_event()
    tofu.MainLoop.begin_round(self)
    
    for pymedia_sound_player in pymedia_sound_players:
      if not pymedia_sound_player.isPlaying():
        pymedia_sound_player.stop()
        pymedia_sound_players.remove(pymedia_sound_player)
        break # breaks because list has been changed
      
class Level(tofu.Level):
  def __init__(self):
    tofu.Level.__init__(self)
    
    self.hour       = 16.0
    self.weather    = "sun"
    
  @tofu.side("single", "client")
  def set_active(self, active):
    old_active = self.active
    
    tofu.Level.set_active(self, active)
    
    if active and not old_active:
      soya.MAIN_LOOP.scenes[0].atmosphere.set_time_and_weather(self.hour, self.weather)
      #soya.MAIN_LOOP.scenes[0].children.remove(soya.MAIN_LOOP.camera)
      #soya.MAIN_LOOP.scenes[0].children.append(soya.MAIN_LOOP.camera)
      
cerealizer.register(Level, soya.cerealizer4soya.SavedInAPathHandler(Level))

