#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Jose 3D
# Copyright (C) 2003-2007 Jean-Baptiste LAMY -- jiba@tuxfamily
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, os.path, sys, glob, distutils.core, distutils.sysconfig
from distutils.core import setup

HERE        = os.path.abspath(os.path.dirname(sys.argv[0]))
HERE_PARENT = os.path.abspath(os.path.join(HERE, os.pardir))

if "--no-lang" in sys.argv:
  sys.argv.remove("--no-lang")
  no_lang = 1
else: no_lang = 0

data_files = []
def walker(dummy, dir, files):
  #print dummy, dir, files
  if "CVS" in dir: return
  if ".svn" in dir: return
  dir2 = dir[len(HERE):]
  if dir2[0] == "/": dir2 = dir2[1:]
  dir2 = os.path.join("jose3d", dir2)
  files = map(lambda file: os.path.join(dir, file), files)
  files = filter(lambda file: os.path.isfile(file) and (not "CVS" in file) and (not ".svn" in file), files)
  data_files.append((dir2, files))
  #data_files.append((dir2, map(lambda file: os.path.join(dir, file), files)))
  
for path in ["animated_models", "images", "materials", "models", "worlds", "sounds", "fonts", "levels", "tofu", "save"]:
  path = os.path.abspath(os.path.join(HERE, path))
  os.path.walk(path, walker, None)
  

if not no_lang:
  data_files = data_files + [
    (os.path.join("jose3d", os.path.dirname(mo_file)), [mo_file])
    for mo_file
    in  glob.glob(os.path.join(".", "locale", "*", "LC_MESSAGES", "*"))
    if not "CVS" in mo_file
    ]

distutils.core.setup(
  name         = "Jose3D",
  version      = "0.1",
  license      = "GPL",
  description  = "A 3D action game.",
  long_description  = """A 3D game in which you play José Bové, candidate for being the French president in 2007.""",
  author       = "Jiba (LAMY Jean-Baptiste)",
  author_email = "jibalamy@free.fr",
  url          = "http://jose3d.org",
  classifiers  = [
  "Topic :: Games/Entertainment",
  "Topic :: Games/Entertainment :: Arcade",
  "Topic :: Multimedia :: Graphics :: 3D Rendering",
  "Natural Language :: French",
  "Programming Language :: Python",
  "Intended Audience :: End Users/Desktop",
  "License :: OSI Approved :: GNU General Public License (GPL)",
  ],
  
  
  scripts      = ["jose3d"],
  package_dir  = {"jose3d" : ""},
  packages     = ["jose3d", "jose3d.tofu"],
  
  data_files   = data_files,
  )

